/* See bmpcount/doc/COPYING for copyright/licensing information. */
/* cestan@cs.ucsd.edu 10/22/2001 */

#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include "multiresbmp.h"
#include "../readwrite/assume.h"

static const double bitsetratio = 0.93105; //(1-1/exp(rhomax)) rhomax = 2.6744

mrb::mrb(int bb, int cc, int memory, int flowIDsize):hash(flowIDsize){
  int i;
  b=bb;
  c=cc;
  assume(b>1,"Components must have at least two bits\n");
  assume(c>0,"The number of components must be positive\n");
  lastb=(memory>0?memory-(c-1)*b:b*2);
  assume3(lastb>=b*2,"Not enough memory for last component: b=%d c=%d mem=%d",
	  b,c,memory);
  assume2(log(b)+log(2)*c<32*log(2),
	  "Multiresolition bitmap too fine for 32 bit hash: b=%d c=%d\n",b,c);
  if(log(b)+log(2)*c>27*log(2)){
    fprintf(stderr,"Unsafe mrb for a 32 bit hash: b=%d c=%d\n",b,c);
  }
  setmax=(int)(b*bitsetratio+0.5);
  for(i=0;i<c;i++){offsets[i]=i*b;}
  offsets[c]=offsets[c-1]+lastb;
  bmp=allocbitmap(offsets[c]);
  assumenotnull(bmp);
  fillzero(bmp,offsets[c]);
}

mrb::~mrb(){
  free(bmp);
}


void mrb::new_packet(byte* flowID, bitmap components){
  uint32bit key=hash.compute(flowID);
  int pos,crt;
  for(crt=0;crt<c-1;crt++){
    if(!(key&1)){/*this is the component*/
      key=key>>1;
      break;
    } else {
      key=key>>1;
    }
  }
  if(crt<c-1){
    pos=key%b;
    setbit(pos+offsets[crt],components);
  } else {
    pos=key%lastb;
    setbit(pos+offsets[c-1],components);
  }
}

int mrb::estimate(bitmap components){
  int base,i,factor,z,j;
  double m;
  factor=1;
  for(base=0;base<c-1;base++){
    z=countzerobits(components,offsets[base],offsets[base]+b);
    if(b-z <= setmax){
      break;
    } else {
      factor=factor*2;
    }
  }
  m=0;
  for(i=base;i<c-1;i++){
    z=countzerobits(components,offsets[i],offsets[i]+b);
    assume4(z!=0,"Bug or statistical anomaly: full component found after base\n b=%d c=%d base=%d crt=%d\n",b,c,base,i);
    m+=b*(log(b)-log(z));
  }
  z=countzerobits(components,offsets[c-1],offsets[c-1]+lastb);
  if(z==0){
    return -1; // Finest component totally full
  } else {
    m+=lastb*(log(lastb)-log(z));
  }
  return (int)(factor*m+0.5);
}

void mrb::reset(bitmap components){
  fillzero(components,offsets[c]);
}

