/* See bmpcount/doc/COPYING for copyright/licensing information. */
/* cestan@cs.ucsd.edu 05/15/2002 */

/* The multiresolution bitmap can operate in two modes: normal and
   split. In split mode you can use a single mrb object to implement
   multiple multiresolution bitmaps with the same configuration but
   different actual bitmaps. In split mode you have to explicitly pass
   the mrb object the bitmap with the components because it does not
   know on its own which "instance" you want to work with. The split
   mode is an optimization to avoid allocating a separate hash (and
   the rest of the overhead) for each instance. */

#include "bitmap.h"
#include "h3.h"
#include "../datatypes/typedefs.h"

#ifndef MRB_H
#define MRB_H

#define MAX_COMPONENTS  32

class mrb{
 public:
  mrb(int b, int c, int memory, int flowIDsize);
  ~mrb();
  void new_packet(byte* flowID){new_packet(flowID,bmp);}
  void new_packet(byte* flowID, bitmap components);
  int estimate(){return estimate(bmp);}
  int estimate(bitmap components);
  void reset(){reset(bmp);}
  void reset(bitmap components);
 private:
  int b;     // The size of a normal component
  int c;     // The number of components
  int lastb; // The size of the last component
  int setmax;// Used to select the base component
  bitmap bmp;// The actual bits
  h3 hash;   // The hash function
  int offsets[MAX_COMPONENTS+1]; // The bit offsets of the components
};

#endif //MRB_H
