/* See bmpcount/doc/COPYING for copyright/licensing information. */
/* cestan@cs.ucsd.edu 10/22/2001 */

#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include "virtbmp.h"
#include "../readwrite/assume.h"

vbmp::vbmp(int bb, double factor, int flowIDsize):hash(flowIDsize){
  b=bb;
  assume(b>1,"Bitmaps must have at least two bits\n");
  assume2(log(b)+log(factor)<32*log(2),
	  "Bitmap too fine for 32 bit hash: b=%d c=%f\n",b,factor);
  if(log(b)+log(factor)>27*log(2)){
    fprintf(stderr,"Unsafe bitmap for a 32 bit hash: b=%d f=%f\n",b,factor);
  }
  bmp=allocbitmap(b);
  assumenotnull(bmp);
  fillzero(bmp,b);
  range=(unsigned)(b*factor);
}

vbmp::~vbmp(){
  free(bmp);
}


void vbmp::new_packet(byte* flowID){
  int pos=hash.compute(flowID)%range;
  if(pos<b){
    setbit(pos,bmp);
  }
}

int vbmp::estimate(){
  int z,j;
  double m;
  z=0;
  for(j=0;j<b;j++){
    if(!getbit(j,bmp)){
      z++;
    }
  }
  assume2(z!=0,"Virtual bitmap full b=%d range=%d\n",b,range);
  return (int)((range*(log(b)-log(z)))+0.5);
}

void vbmp::reset(){
  fillzero(bmp,b);
}

