/* See bmpcount/doc/COPYING for copyright/licensing information. */
/* cestan@cs.ucsd.edu 06/27/2003 */

#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include "compound.h"
#include "../readwrite/assume.h"

compound::compound(thing*t0, thing*t1, thing*t2, thing*t3, thing*t4,
		   thing*t5, thing*t6, thing*t7, thing*t8, thing*t9,
		   thing*t10, thing*t11, thing*t12, thing*t13, thing*t14,
		   thing*t15, thing*t16, thing*t17, thing*t18, thing*t19){
  int i;
  assume(t0!=NULL, "Each record needs at least one object\n");
  things[0]=t0;things[1]=t1;things[2]=t2;things[3]=t3;things[4]=t4;
  things[5]=t5;things[6]=t6;things[7]=t7;things[8]=t8;things[9]=t9;
  things[10]=t10;things[11]=t11;things[12]=t12;things[13]=t13;things[14]=t14;
  things[15]=t15;things[16]=t16;things[17]=t17;things[18]=t18;things[19]=t19;
  numthings=MAX_COMPOUND_SIZE;
  /* First we count the number of things we actually have. */
  for(i=0;i<MAX_COMPOUND_SIZE;i++){
    if(things[i]==NULL && numthings==MAX_COMPOUND_SIZE){
      numthings=i;
      break;
    }
  }
  for(;i<MAX_COMPOUND_SIZE;i++){
    assume(things[i]==NULL, "NULL pointers should not be followed by non NULL pointers in compound constructor\n");
  }
  /* Now we initialize those other auxiliary structures. */
  offsets[0]=0;
  maxhrsize=numthings-1; /* this many separating spaces */
  for(i=0;i<numthings;i++){
    offsets[i+1]=offsets[i]+things[i]->getbinsize();
    maxhrsize+=things[i]->getmaxhrsize();
  }
  for(;i<MAX_COMPOUND_SIZE;i++){
    offsets[i+1]=offsets[i];
  }
  assume2(maxhrsize<BUFFERSIZE-1,
	  "Size of the record %u not smaller than buffer size %u.\n",
	  maxhrsize,BUFFERSIZE);
}

void compound::print (buffer& b, const  byte* data){
  for(int i=0;i<numthings-1;i++){
    things[i]->print(b,data+offsets[i]);
    b.print_char('\t');
  }
  things[numthings-1]->print(b,data+offsets[numthings-1]);
}

int compound::parse (buffer& b, byte* data){
  for(int i=0;i<numthings-1;i++){
    if(!things[i]->parse(b,data+offsets[i])){
      return 0;
    }
    b.skipspaces();
  }
  if(!things[numthings-1]->parse(b,data+offsets[numthings-1])){
    return 0;
  }
  return 1;
}

void compound::printname(buffer& b){
  for(int i=0;i<numthings-1;i++){
    things[i]->printname(b);
    b.print_char('\t');
  }
  things[numthings-1]->printname(b);
}

void compound::prettyprint(buffer& b, const byte* data){
  for(int i=0;i<numthings;i++){
    things[i]->prettyprint(b,data+offsets[i]);
  }
}
 
void compound::prettyprintname(buffer& b){
  for(int i=0;i<numthings;i++){
    things[i]->prettyprintname(b);
  }
}
