/* See bmpcount/doc/COPYING for copyright/licensing information. */
/* cestan@cs.ucsd.edu 07/10/2002 */

#include "timestamp.h"

/* We have a slight problem with the timestamps in the traces: the
   number of digits after the dot varies. So our parse routine will
   accept any number of digits after the dot, but our print routine
   will always print 9 (because we count nanoseconds). */

/* We need different multipliers to convert to nanoseconds based on
   the number of digits we have after the decimal dot. It is faster to
   look the multiplier up than to do exponentiation.*/
static long multiplier[10]={0, /*we never use multuiplier[0] */ 
		     100000000,
		     10000000,
		     1000000,
		     100000,
		     10000,
		     1000,
		     100,
		     10,
		     1};

void timestamp::print(buffer& b, const  byte* data){
  b.offset+=sprintf(b.buff+b.offset,"%llu.%09llu",(*(uint8*)(data))/1000000000,
		    (*(uint8*)(data))%1000000000);
}

int timestamp::parse(buffer& b, byte* data){
  uint8 seconds;
  int dotposition;
  if(!b.parse_unsigned8(&seconds)){
    b.errorstring="Cannot parse seconds for timestamp.";
    return 0;
  }
  if(seconds > 18446744072ull){
    b.errorstring="Too many seconds in timestamp.";
    return 0;
  }
  if(!b.parse_char('.',"Decimal dot missing from timestamp.")){
    return 0;
  }
  dotposition=b.offset;
  if(!b.parse_unsigned8((uint8*)(data))){
    b.errorstring="Cannot parse fractional seconds for timestamp.";
    return 0;
  }
  if(b.offset-dotposition>9){
    b.errorstring="Too many digits after decimal dot for timestamp.";
    return 0;
  }
  (*((uint8*)(data)))=1000000000*seconds+
     (*((uint8*)(data)))*multiplier[b.offset-dotposition];
}

