/* See bmpcount/doc/COPYING for copyright/licensing information. */
/* cestan@cs.ucsd.edu 10/07/2003 */

/* Our add_delete_hash is a hash_map you can add entries to and delete
   entries from. Unless there are less than 9,000 entries or there
   were more deletions than additions recently, the overhead is 1.25
   to 1.5 pointers per entry and only 0% to 7.2% of the allocated
   entries is unused enteries. The iterator traverses the entries in
   random order and allows the deletion of selected entries (but
   deleting an entry does not free up its memory). */

#ifndef ADD_DELETE_HASH_H
#define ADD_DELETE_HASH_H

#include "hash_map.h"

class add_delete_hash:public hash_map{
  friend class add_delete_hash_iterator;
 public:
  add_delete_hash(unsigned int keysize, unsigned int valuesize);
  unsigned int size(){return numentriesused;}
  hash_map_iterator* build_iterator();
 protected:
  int numentriesused;
  struct entrylist* unused_entries;
  byte* add_key(const byte* key,entrylist** crtp);
  struct entrylist* add_block(unsigned int numentr);
};

class add_delete_hash_iterator:public hash_map_iterator{
 public:
  add_delete_hash_iterator(add_delete_hash& map);
  int at_end(){return crttableentry>=m->ts;}
  void operator++();
  void delete_crt_entry(); /* Deletes current entry and moves iterator
			      to the next one. */
 protected:
  int crttableentry;
  add_delete_hash* m;
};


#endif /* ADD_DELETE_HASH_H */
