/* See bmpcount/doc/COPYING for copyright/licensing information. */
/* cestan@cs.ucsd.edu 06/20/2003 */

/* Our hash_map is an associative memory that maps from fixed size
   keys to fixed size values. The size of the keys and that of the
   values are passed on at initialization. Key value pairs are stored
   contiguously (key first) in entries that are never moved once they
   are allocated. When a new entry is created its value is initialized
   to all 0s. Keys must be at least 1 byte. Once an entry is created
   it is never moved as the hash grows, so it is safe to store
   pointers to it. */

#ifndef HASHMAP_H
#define HASHMAP_H

#include "../datatypes/typedefs.h"
#include "jiffyhash.h"

struct entrylist {
  struct entrylist* next;
  byte entry[1];
};
/* Actually entries are larger than this struct: they fit the next
   pointer plus the entry itself (the key plus the value). */

class hash_map{
  friend class hash_map_iterator;
 public:
  hash_map(unsigned int keysize, unsigned int valuesize);
  ~hash_map();
  int has(const byte* key); /* Checks existence of key */
  void insert(const byte* key); /* Inserts key */
  byte* operator[](const byte* key);
  /* If the key has no entry associated, it creates one */
  virtual unsigned int size()=0;
  unsigned int keysize(){return ks;}
  unsigned int valuesize(){return vs;}
  virtual hash_map_iterator* build_iterator()=0;
 protected:
  jiffyhash hashfun; /* hash function */
  const unsigned int ks; /* key size */
  const unsigned int vs; /* value size */
  struct entrylist** table;
  unsigned int ts; /* table size */
  struct entrylist* blocks[166]; /* these hold the entries */
  const unsigned int es; /* size of an element in a block */
  unsigned int numblocks;
  virtual byte* add_key(const byte* key,entrylist** crtp)=0;
};

class hash_map_iterator{
 public:
  hash_map_iterator(hash_map& map);
  virtual int at_end()=0;
  const byte* key(){return crtentry->entry;}
  byte* value(){return ((byte*)(crtentry->entry)+ks);}
  virtual void operator++()=0;
 protected:
  struct entrylist* crtentry;
  const unsigned int es;
  const unsigned int ks;
};


#endif /* HASHMAP_H */
