/* See AutoFocus/doc/COPYING for copyright/licensing information. */
/* cestan@cs.ucsd.edu 06/20/2003 */

#include <stdlib.h>
#include "jiffyhash.h"
#include "../readwrite/assume.h"

static void initialize_offsets(unsigned int* shiftoffsets, int ks);
static int good_enough(unsigned int* shiftoffsets, int ks);
// Checks whether the offsets provide enough randomness

jiffyhash::jiffyhash(unsigned int keysize):ks(keysize){
  shiftoffsets=(unsigned int*)malloc(sizeof(unsigned int)*ks);
  assume(shiftoffsets!=NULL,"Cannot allocate memory for hash function\n");
  assume(sizeof(hash_key)==4,"Internal hash key has to be 4 bytes\n");
  while(!good_enough(shiftoffsets,ks)){
    initialize_offsets(shiftoffsets,ks);
  }
}

jiffyhash::~jiffyhash(){
  free(shiftoffsets);
}

static int good_enough(unsigned int* shiftoffsets, int ks){
  if(ks==1){return 1;}
  if(ks==2){
    if(abs((int)shiftoffsets[0]-int(shiftoffsets[1]))>7){
      return 1;
    } else {
      return 0;
    }
  }
  if(ks<=4){ // shift positions must be nonoverlapping
    if(abs((int)shiftoffsets[0]-int(shiftoffsets[1]))>7 &&
       abs((int)shiftoffsets[2]-int(shiftoffsets[1]))>7 &&
       abs((int)shiftoffsets[0]-int(shiftoffsets[2]))>7){
      return 1;
    } else {
      return 0;
    }
  } else { // shift positions must cover all bits (this means for 4
	   // bytes we have only premutations
    unsigned int mask =0;
    for(int i=0;i<ks;i++){
      mask|=0xFF<<shiftoffsets[i];
    }
    if(mask==0xFFFFFFFF){
      return 1;
    } else {
      return 0;
    }
  }
}

static void initialize_offsets(unsigned int* shiftoffsets, int ks){
  for(int i=0;i<ks;i++){
    shiftoffsets[i]=(rand()%5639)%25;
  }
  if(ks>4){
    shiftoffsets[(rand()%5639)%ks]=0;
    shiftoffsets[(rand()%5639)%ks]=24;
  }
}


