/* See bmpcount/doc/COPYING for copyright/licensing information. */
/* cestan@cs.ucsd.edu 10/28/2001 */

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include "../getopt/getopt.h"
#include "../bitmaps/adaptivebmp.h"
#include "../datatypes/ip.h"
#include "../datatypes/port.h"
#include "../datatypes/proto.h"
#include "../datatypes/timestamp.h"
#include "../datatypes/compound.h"
#include "../readwrite/asciifile.h"
#include "../readwrite/assume.h"

/* This is the configuration of the adaptive bitmap. Using incorrect
   values for rhomaxbig and rhominbig can silently ruin the accuracy
   of the results. Use bmpcount/scripts/configureabmp.pl to get the
   correct values. */
int b=64;
int c=19;
int bigb=15063;
int spanbig=9;
int memory=16384;
double rhominbig=0.007148;
double rhomaxbig=9.4286;

char* tracefile="-";
int seed=0;
long long int interval=5;

static struct option long_options[] =
{
  {"trace",1,0,0}, // the trace file (default stdin)
  {"seed",1,0,0}, // the random seed
  {"interval",1,0,0}, // the measurement interval in seconds
  {0,0,0,0}
};

int main(int argc,char* argv[]){
  int tick=0;
  /*options stuff*/
  int option_index=0;
  int ii;

  /* parsing the command line options*/
  while((ii=getopt_long_only(argc,argv,"",long_options,&option_index))!=-1){
    if (ii!=0){
      fprintf (stderr,"Problem with options %c .\n",ii);
      exit(1);
    }
    switch (option_index) {
    case 0:
      tracefile=optarg;
      break;
    case 1:
      seed=atoi(optarg);
      assume(seed>=0,"Seed must be positive\n");
      break;
    case 2:
      interval=atoi(optarg);
      assume(interval>0,"Interval size must be above 0\n");
      break;
    default:
      fprintf(stderr,"Major command line argument parsing error.\n");
      abort();
    }
  }
  srand(seed);

  ip srcip("Source IP");
  ip destip("Dest. IP");
  port srcport("SrcP");
  port destport("DestP");
  proto protocol("Pro");
  timestamp ts("TimeStamp");
  compound traceline(&ts,&srcip,&destip,&protocol,&srcport,&destport);
  compound flowID(&srcip,&destip,&protocol,&srcport,&destport);
  unsigned idoffset=ts.getbinsize();
  abmp bmp(b,c,bigb,spanbig,memory,rhominbig,rhomaxbig,flowID.getbinsize());
  /* The timestamp with the end of the current interval. */
  byte endofint[8]={'\0','\0','\0','\0','\0','\0','\0','\0'};
  byte *linebytes; /* The current packet parsed */
  asciifilereader trace(tracefile,traceline);
  while(linebytes=trace.getnextrecord()){
    if(!ts.isbefore(linebytes,endofint)){
      if(*((uint8*)endofint)==0){ /* First interval */
	copy(linebytes,endofint,8);
	*((uint8*)endofint)+=interval*1000000000;
      } else {
	tick++;
	ts.setseconds(endofint,ts.getseconds(endofint)+interval);
	printf("%3u %6u\n",tick,bmp.estimate());
	bmp.reset();
      }
    }
    bmp.new_packet(linebytes+idoffset);
  }
}
