/* See bmpcount/doc/COPYING for copyright/licensing information. */
/* cestan@cs.ucsd.edu 08/23/2001 */

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include "../getopt/getopt.h"
#include "../bitmaps/virtbmp.h"
#include "../datatypes/ip.h"
#include "../datatypes/port.h"
#include "../datatypes/proto.h"
#include "../datatypes/timestamp.h"
#include "../datatypes/compound.h"
#include "../readwrite/asciifile.h"
#include "../readwrite/assume.h"

char* tracefile="-";
int bmsize=1000;
double factor=1;
int seed=0;
long long int interval=5;

static struct option long_options[] =
{
  {"trace",1,0,0},    // the trace file (default stdin)
  {"bmsize",1,0,0},   // the size of our bitmap
  {"factor",1,0,0},   // the ratio between the virtual and actual table size
  {"seed",1,0,0},     // the random seed
  {"interval",1,0,0}, // the measurement interval in seconds
  {0,0,0,0}
};

int main(int argc,char* argv[]){
  int tick=0;
  timestamp nextreport=0;
  int i;
  /*options stuff*/
  int option_index=0;
  int c;

  /* parsing the command line options*/
  while((c=getopt_long_only(argc,argv,"",long_options,&option_index))!=-1){
    if (c!=0){
      fprintf (stderr,"Problem with options %c .\n",c);
      exit(1);
    }
    switch (option_index) {
    case 0:
      tracefile=optarg;
      break;
    case 1:
      bmsize=atoi(optarg);
      assume(bmsize>0,"Bitmap size must be positive\n");
      break;
    case 2:
      factor=atof(optarg);
      assume(factor>=1,"Factor must be at least 1\n");
      break;
    case 3:
      seed=atoi(optarg);
      assume(seed>=0,"Seed must be positive\n");
      break;
    case 4:
      interval=atoi(optarg);
      assume(interval>0,"Interval size must be at least 1\n");
      break;
    default:
      fprintf(stderr,"Major command line argument parsing error.\n");
      abort();
    }
  }

  srand(seed);
  ip srcip("Source IP");
  ip destip("Dest. IP");
  port srcport("SrcP");
  port destport("DestP");
  proto protocol("Pro");
  timestamp ts("TimeStamp");
  compound traceline(&ts,&srcip,&destip,&protocol,&srcport,&destport);
  compound flowID(&srcip,&destip,&protocol,&srcport,&destport);
  unsigned idoffset=ts.getbinsize();
  vbmp bmp(bmsize,factor,flowID.getbinsize());
  /* The timestamp with the end of the current interval. */
  byte endofint[8]={'\0','\0','\0','\0','\0','\0','\0','\0'};
  byte *linebytes; /* The current packet parsed */
  asciifilereader trace(tracefile,traceline);
  while(linebytes=trace.getnextrecord()){
    if(!ts.isbefore(linebytes,endofint)){
      if(*((uint8*)endofint)==0){ /* First interval */
	copy(linebytes,endofint,8);
	*((uint8*)endofint)+=interval*1000000000;
      } else {
	tick++;
	ts.setseconds(endofint,ts.getseconds(endofint)+interval);
	printf("%3u %6u\n",tick,bmp.estimate());
	bmp.reset();
      }
    }
    bmp.new_packet(linebytes+idoffset);
  }
}
