/* See bmpcount/doc/COPYING for copyright/licensing information. */
/* cestan@cs.ucsd.edu 12/9/2003 */

/* This program counts the exact number of flows associated with each
   source */

#include <stdio.h>
#include <stdlib.h>
#include "../getopt/getopt.h"
#include "../hash_map/add_delete_hash.h"
#include "../datatypes/ip.h"
#include "../datatypes/port.h"
#include "../datatypes/proto.h"
#include "../datatypes/timestamp.h"
#include "../datatypes/compound.h"
#include "../readwrite/asciifile.h"
#include "../readwrite/assume.h"

char* tracefile="-";

static struct option long_options[] =
{
  {"trace",1,0,0}, // the trace file (default stdin)
  {0,0,0,0}
};

int main(int argc,char* argv[]){
  int tick=0;
  /*options stuff*/
  int option_index=0;
  int ii;

  /* parsing the command line options*/
  while((ii=getopt_long_only(argc,argv,"",long_options,&option_index))!=-1){
    if (ii!=0){
      fprintf (stderr,"Problem with options %c .\n",ii);
      exit(1);
    }
    switch (option_index) {
    case 0:
      tracefile=optarg;
      break;
    default:
      fprintf(stderr,"Major command line argument parsing error.\n");
      abort();
    }
  }

  ip srcip("Source IP");
  ip destip("Dest. IP");
  port srcport("SrcP");
  port destport("DestP");
  proto protocol("Pro");
  timestamp ts("TimeStamp");
  compound traceline(&ts,&srcip,&destip,&protocol,&srcport,&destport);
  compound flowID(&srcip,&destip,&protocol,&srcport,&destport);
  unsigned idoffset=ts.getbinsize();
  unsigned srcipoffset=idoffset;
  add_delete_hash flowspersource(srcip.getbinsize(),sizeof(unsigned int));
  add_delete_hash flows(flowID.getbinsize(),0);
  byte *linebytes; /* The current packet parsed */
  asciifilereader trace(tracefile,traceline);
  while(linebytes=trace.getnextrecord()){
    if(flows.has(linebytes+idoffset)){
      continue;
    }
    flows.insert(linebytes+idoffset);
    (*(unsigned int *)(flowspersource[linebytes+srcipoffset]))++;
  }
  buffer b;
  for(add_delete_hash_iterator i(flowspersource);!i.at_end();++i){
    b.offset=0;
    srcip.print(b,i.key());
    b.print_char('\0');
    printf("%s\t%u\n",b.buff,*((unsigned int *)i.value()));
  }
}
