/* See bmpcount/doc/COPYING for copyright/licensing information. */
/* cestan@cs.ucsd.edu 05/14/2001 */


#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include "../getopt/getopt.h"
#include "../hash_map/add_delete_hash.h"
#include "../datatypes/ip.h"
#include "../datatypes/port.h"
#include "../datatypes/proto.h"
#include "../datatypes/timestamp.h"
#include "../datatypes/compound.h"
#include "../readwrite/asciifile.h"
#include "../readwrite/assume.h"
#include "../bitmaps/multiresbmp.h"
#include "../bitmaps/h3.h"

char* tracefile="-";
int dbmsize=32;
int b=64;
int c=11;
int memory=768;
int trigger=4;
int seed=0;

static struct option long_options[] =
{
  {"trace",1,0,0}, // the trace file (default stdin)
  {"dbmsize",1,0,0}, // the size of our per source IP direct bitmaps
  {"b",1,0,0}, // the b of the multiresolution bitmap
  {"c",1,0,0}, // the c of the multiresolution bitmap
  {"memory",1,0,0}, // the memory given to the mrb (from this we get the size of the last component)
  {"trigger",1,0,0}, // when this many bits are set, we allocate the multiresolution bitmap for that source
  {"seed",1,0,0}, // the random seed
  {0,0,0,0}
};

int main(int argc,char* argv[]){
  /*options stuff*/
  int option_index=0;
  int i;

  /* parsing the command line options*/
  while((i=getopt_long_only(argc,argv,"",long_options,&option_index))!=-1){
    if (i!=0){
      fprintf (stderr,"Problem with options %c .\n",c);
      exit(1);
    }
    switch (option_index) {
    case 0:
      tracefile=optarg;
      break;
    case 1:
      dbmsize=atoi(optarg);
      assume(dbmsize>1,"The direct bitmap has to be at least 2");
      break;
    case 2:
      b=atoi(optarg);
      assume(b>1,"There must be at least two bits per component");
      break;
    case 3:
      c=atoi(optarg);
      assume(c>0,"Need at least one component in mrb");
      break;
    case 4:
      memory=atoi(optarg);
      assume(memory>0,"Per mrb memory must be positive");
      break;
    case 5:
      trigger=atoi(optarg);
      assume(trigger>0,"Trigger must be positive");
      break;
    case 6:
      seed=atoi(optarg);
      assume(seed>=0,"Seed must be positive");
      break;
    default:
      fprintf(stderr,"Major command line argument parsing error.\n");
      abort();
    }
  }
  assume2(trigger<dbmsize,"Trigger value (%d) must be smaller than ",trigger,dbmsize);
  srand(seed);

  ip srcip("Source IP");
  ip destip("Dest. IP");
  port srcport("SrcP");
  port destport("DestP");
  proto protocol("Pro");
  timestamp ts("TimeStamp");
  compound traceline(&ts,&srcip,&destip,&protocol,&srcport,&destport);
  compound flowID(&srcip,&destip,&protocol,&srcport,&destport);
  unsigned idoffset=ts.getbinsize();
  unsigned srcipoffset=idoffset;
  add_delete_hash allsrcs(srcip.getbinsize(),bitmapsize(dbmsize));
  add_delete_hash largesrcs(srcip.getbinsize(),bitmapsize(memory));
  mrb mrbmp(b,c,memory,flowID.getbinsize());
  h3 hash(flowID.getbinsize());
  bitmap bmp;
  unsigned index;
  byte *linebytes; /* The current packet parsed */
  asciifilereader trace(tracefile,traceline);
  while(linebytes=trace.getnextrecord()){
    bmp=(bitmap)allsrcs[linebytes+srcipoffset];
    index=hash.compute(linebytes+idoffset)%dbmsize;
    if(getbit(index,bmp)){
      continue;
    }
    if(countzerobits(bmp,0,dbmsize)+trigger==dbmsize){ // we go to the mrb
      mrbmp.new_packet(linebytes+idoffset,
		       (bitmap)largesrcs[linebytes+srcipoffset]);
    } else { // we stay in the small direct bitmap
      setbit(index,bmp);
    }
  } 
  buffer b;
  int smallcounter,bigcounter;
  smallcounter=bigcounter=0;
  double estimate;
  for(add_delete_hash_iterator i(allsrcs);!i.at_end();++i){
    smallcounter++;
    b.offset=0;
    srcip.print(b,i.key());
    b.print_char('\0');
    bmp=(bitmap)i.value();
    estimate=dbmsize*(log(dbmsize)-log(countzerobits(bmp,0,dbmsize)));
    if(largesrcs.has(i.key())){
	estimate+=(double)(dbmsize*mrbmp.estimate((bitmap)largesrcs[i.key()]))
	  /(dbmsize-trigger);
	bigcounter++;
      }
    printf("%s\t%u\n",b.buff,(int)(estimate+0.5));
  }
  fprintf(stderr,"%d\t%d\n",smallcounter,bigcounter);
}
