/* See bmpcount/doc/COPYING for copyright/licensing information. */
/* cestan@cs.ucsd.edu 12/05/2002 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "basicfilereader.h"
#include "compressed.h"

basicfilereader::basicfilereader(const char*filename){
  if (strcmp(filename,"-")==0) {
    file=stdin;
  } else {
    if (iscompressed(filename)) {
      file=readopencompressed(filename);
    } else {
      file=fopen(filename,"r");
    }
    if (file==NULL){
      fprintf(stderr,"Cannot read file %s",filename);
      perror("");
      exit(1);
    }
  }
  linenumber=0;
  b.buff[0]='\0';
  b.offset=0;
  fname=filename;
}

basicfilereader::~basicfilereader(){
  if (strcmp(fname,"-")!=0) {
    fclose(file);
  }
}

int basicfilereader::getnextline(int skipcomments){
  if (!fgets(b.buff,BUFFERSIZE,file)){
    return 0;
  }
  linenumber++;
  if(strlen(b.buff)==BUFFERSIZE-1){
    fprintf(stderr,"Line %u too long.\n",linenumber);
    printbuffererror();
  }
  // Overwrite the final '\n'
  if(strlen(b.buff)>0 && b.buff[strlen(b.buff)-1]=='\n'){
    b.buff[strlen(b.buff)-1]='\0';
  }
  if(b.buff[0]=='!' || b.buff[0] =='#' || b.buff[0] == ';'){
    /* Skip this line, it's a comment */
    if(skipcomments){
      return getnextline();
    }
  }
  b.errorstring="";
  b.offset=0; 
  // Skip spaces at start
  while(b.buff[b.offset]==' ' || b.buff[b.offset]=='\t'){
    b.offset++;
  }
  return 1;
}

void basicfilereader::printbuffererror(char* errorstr){
  if(errorstr!=NULL){
    b.errorstring=errorstr;
  }
  fprintf(stderr,"%s\n",b.buff);
  memset(b.buff,(int)(' '),b.offset);
  b.buff[b.offset]='^';
  b.buff[b.offset+1]='\0';
  fprintf(stderr,"%s\n",b.buff);
  fprintf(stderr,"%s\n",b.errorstring);
  exit(1);
}

