/* See bmpcount/doc/COPYING for copyright/licensing information. */
/* cestan@cs.ucsd.edu 07/10/2002 */

/* Implementation of the buffer class */

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include "buffer.h"
#include "assume.h"

buffer::buffer(void){
  assume(sizeof(uint1)==1,"sizeof(uint1)!=1\n");
  assume(sizeof(uint2)==2,"sizeof(uint2)!=2\n");
  assume(sizeof(uint4)==4,"sizeof(uint4)!=4\n");
  assume(sizeof(uint8)==8,"sizeof(uint8)!=8\n");
  offset=0;
  errorstring=NULL;
}

int buffer::parse_unsigned1(uint1* location){
  int crt,originaloffset;
  uint2 result;
  originaloffset=offset;
  while(isdigit(buff[offset])){
    offset++;
  }
  if(offset == originaloffset){
    errorstring="No digits found where unsigned expected.";
    return 0;
  }
  if(offset - originaloffset > 3){
    errorstring="Too many digits for one byte unsigned.";
    offset=originaloffset;
    return 0;
  }
  result=0;
  for(crt=originaloffset;crt<offset;crt++){
    result=result*10;
    result+=buff[crt]-('0');
  }
  if(result>255){
    errorstring="Value too large for one byte unsigned.";
    return 0;
  }
  *location=(uint1)result;
  return 1;
}

int buffer::parse_unsigned2(uint2* location){
  int crt,originaloffset;
  uint4 result;
  originaloffset=offset;
  while(isdigit(buff[offset])){
    offset++;
  }
  if(offset == originaloffset){
    errorstring="No digits found where unsigned expected.";
    return 0;
  }
  if(offset - originaloffset > 5){
    errorstring="Too many digits for two byte unsigned.";
    offset=originaloffset;
    return 0;
  }
  result=0;
  for(crt=originaloffset;crt<offset;crt++){
    result=result*10;
    result+=buff[crt]-('0');
  }
  if(result>65535){
    errorstring="Value too large for two byte unsigned.";
    return 0;
  }
  *location=(uint2)result;
  return 1;
}

int buffer::parse_unsigned4(uint4* location){
  int crt,originaloffset;
  uint8 result;
  originaloffset=offset;
  while(isdigit(buff[offset])){
    offset++;
  }
  if(offset == originaloffset){
    errorstring="No digits found where unsigned expected.";
    return 0;
  }
  if(offset - originaloffset > 10){
    errorstring="Too many digits for four byte unsigned.";
    offset=originaloffset;
    return 0;
  }
  result=0;
  for(crt=originaloffset;crt<offset;crt++){
    result=result*10;
    result+=buff[crt]-('0');
  }
  if(result>4294967295ull){
    errorstring="Value too large for four byte unsigned.";
    return 0;
  }
  *location=(uint4)result;
  return 1;
}

int buffer::parse_unsigned8(uint8* location){
  int crt,originaloffset;
  uint8 result;
  originaloffset=offset;
  while(isdigit(buff[offset])){
    offset++;
  }
  if(offset == originaloffset){
    errorstring="No digits found where unsigned expected.";
    return 0;
  }
  if(offset - originaloffset > 20){
    errorstring="Too many digits for eight byte unsigned.";
    offset=originaloffset;
    return 0;
  }
  result=0;
  for(crt=originaloffset;crt<offset;crt++){
    if(result>1844674407370955160ull && buff[crt]>('5')){
      errorstring="Value too large for eight byte unsigned.";
      offset=originaloffset;
      return 0;
    }
    result=result*10;
    result+=buff[crt]-('0');
  }
  *location=result;
  return 1;
}

static char expchar[]=" KMGTPE";
/*bytes, kilobytes, megabytes, gigabytes, terabytes, petabytes, exabytes*/

void buffer::print_traffic_threedigits(uint8 value){
  int exponent=0;
  while(value>=102400){
    value=value/1024;
    exponent++;
  }
  if(value<1000){
    print_unsigned8(value);
    if(exponent>0){
      buff[offset]=expchar[exponent];
      offset++;
    }
  } else {
    double v=value/1024.0;
    if(v<9.995){
      sprintf(buff+offset,"%1.02f",v);
      offset+=4;
    } else {
      if(v>99.95){
	sprintf(buff+offset,"100");
	offset+=3;
      } else {
	sprintf(buff+offset,"%2.01f",v);
	offset+=4;
      }
    }
    buff[offset]=expchar[exponent+1];
    offset++;
  }
  buff[offset]='\0';
}
