/* See bmpcount/doc/COPYING for copyright/licensing information. */
/* cestan@cs.ucsd.edu 07/10/2002 */

/* The buffer is used for parsing stuff read in from somewhere and
   printing stuff before it's sent to a file or stdout. It knows how
   to handle integers of various lengths. It exposes its internals to
   classes that want to do more complex stuff (i.e. they have to move
   the offset themselves after a read or write). */

#ifndef BUFFER_H
#define BUFFER_H

#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include "../datatypes/typedefs.h"


#define BUFFERSIZE 300

class buffer{
 public:
  char buff[BUFFERSIZE];
  int offset;
  const char* errorstring;
  buffer();
  void print_unsigned1(uint1 value)
    {offset+=sprintf(buff+offset,"%u",(unsigned)value);}
  void print_unsigned2(uint2 value)
    {offset+=sprintf(buff+offset,"%u",(unsigned)value);}
  void print_unsigned4(uint4 value)
    {offset+=sprintf(buff+offset,"%u",value);}
  void print_unsigned8(uint8 value)
    {offset+=sprintf(buff+offset,"%llu",value);}
  void print_char(char c)
    {buff[offset++]=c;}
  void print_string(const char* s)
    {strncpy(buff+offset,s,BUFFERSIZE-offset);
    offset=(strlen(s)>BUFFERSIZE-offset?BUFFERSIZE:offset+strlen(s));}

  int parse_unsigned1(uint1* location);
  int parse_unsigned2(uint2* location);
  int parse_unsigned4(uint4* location);
  int parse_unsigned8(uint8* location);
  void skipspaces(){while(isspace(buff[offset])){offset++;}}
  int parse_char(char c,const char* es)
    {if(buff[offset]==c){offset++;return 1;}else{errorstring=es;return 0;}}
  int parse_char(char c)
    {if(buff[offset]==c){offset++;return 1;}else{return 0;}}
  void skip_if_present(char c){if(buff[offset]==c){offset++;}}
  void print_traffic_threedigits(uint8 value);
  /* Prints a traffic counter using at most 3 digits (converts it to
     kilobytes, megabytes, etc. if necessary) */
};

#endif /* BUFFER_H */
