/* See bmpcount/doc/COPYING for copyright/licensing information. */
/* cestan@cs.ucsd.edu 05/29/2001 */

#include <sys/types.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include "compressed.h"

int iscompressed(const char* filename) {
  if (strcmp(filename+strlen(filename)-3,".gz")==0){
    return 1;
  }
  return 0;
}

FILE* readopencompressed(const char* filename){
  if (strcmp(filename+strlen(filename)-3,".gz")==0){
    int filedes[2];
    pid_t childpid;
    if(pipe(filedes)==-1){
      perror("Error opening pipe");
      return NULL;
    }
    childpid=fork();
    if(childpid==0){
      /* This is the child, we need to start the gunzip */
        /* Redirecting the standard output to the pipe */
      if(dup2(filedes[1],1)==-1){
	perror("Error redirecting standard output");
	exit(1);
      }
      /* we do not need the filedes file descriptors anymore */
      if(close(filedes[0])==-1){
	perror("");
      }
      if(close(filedes[1])==-1){
	perror("");
      }
      execlp("gunzip","gunzip","-c",filename,NULL);
      perror("Cannot execute gunzip");
      exit(1);
    } else {
      /* This is the parent doing the trace analysis */
      FILE* result;
      if (childpid == -1){
	perror("Error forking child process for gunzip");
	return NULL;
      }
      /* The writing end of the pipe *has to* be closed so that the
	 gunzip is the only one that can write to the pipe and we can
	 properly terminate the analysis when the file is over. */
      if(close(filedes[1])==-1){
	perror("");
      }
      result=fdopen(filedes[0],"r");
      if(result == NULL){
	perror("Problem with pipe");
	return NULL;
      }
      return result;
    }
  }
  fprintf(stderr,"Internal error, file %s thought to be compressed.",filename);
  return NULL;
}


