#!/usr/bin/perl

# This script finds the magic constants (bigb/b, the ratio between the
# peak and worst error, where it is attained and the flow density
# rhomaxprime for which it is worth to switch to the next component)
# for an adaptive bitmap for various values of r (the number of
# components replaced by the virtual bitmap.

sub vbmpvariance($$){
  my ($rho,$b)=($_[0],$_[1]);
  return $b*(exp($rho)-1);
}

sub manycomponentsvariance($$$){
  my ($rho,$b,$r)=($_[0],$_[1],$_[2]);
  my $result=0;
  while ($r>0){
    $result+=$b*(exp($rho)-1);
    $rho=$rho/2;
    $r--;
  }
  return $result;
}

# The average (relative) error of the large component of the adaptive bitmap

sub abmperror($$$$){
  my ($rho,$r,$b,$v)=($_[0],$_[1],$_[2],$_[3]);
  my $v0=2*$b*(1-2**(-$r));
  my $rhomrbnext=$rho*$v/2**$r/$v0;
  return sqrt($v*(exp($rho)-1)+$b*(exp($rhomrbnext)-1)+2*$b*(exp($rhomrbnext/2)-1))/$v/$rho*$v0/$b/2;
}

# Computes the error of the multiresolution bitmap component just before the big virtual bitmap

sub abmpbberror($$$$){
  my ($rho,$r,$b,$v)=($_[0],$_[1],$_[2],$_[3]);
  my $v0=2*$b*(1-2**(-$r));
  return (sqrt($b*(exp($rho)-1)+$v*(exp($rho*$v0/2/$v)-1)+$b*2*(exp($rho/2**($r+1))-1))/$rho/$b/2);
}

sub mrberror($){
  my ($rho,$k)=($_[0],2);
  return (sqrt((($k-1)/$k*(exp($rho)+exp($rho/$k)-2)+exp($rho/$k/$k)-1)*($k-1)/$k)/$rho);
}

$b=1; #any positive number should do

$increment=1.00001;

for($r=2;$r<=16;$r++){
  $v0=$v=2*$b*(1-2**(-$r)); #equivalent to just extending the coarsest components
  while(vbmpvariance(1.3372*$v0/$v,$v)>manycomponentsvariance(1.3372,$b,$r) or
	abmperror(2**($r-1)*2.6744*$v0/$v,$r,$b,$v)>mrberror(2.6744) or
	abmpbberror(1.3372,$r,$b,$v)>mrberror(1.3372) or
	abmpbberror(2.6744,$r,$b,$v)>mrberror(2.6744)){
    $v=$v*$increment;
    if($v>100000*$v0){
      print abmperror(2**($r-1)*2.6744*$v0/$v,$r,$b,$v)."\t".(2**($r-1)*2.6744*$v0/$v)."\t".
      (sqrt(vbmpvariance(2**($r-1)*2.6744*$v0/$v,$v))/(2**($r-1)*2.6744*$v0))."\n";
      die;
    }
  }
  if(abmperror(1.3372*$v0/$v,$r,$b,$v)>mrberror(1.3372)){
    print abmperror(1.3372*$v0/$v,$r,$b,$v)."\t".mrberror(1.3372)."\n";
    die;
  }
  if(abmperror(2**($r-1)*2.6744*$v0/$v,$r,$b,$v)>mrberror(2.6744)){
    die;
  }
  $w=$v/$increment;
  print "$r\t$v ";
  if (vbmpvariance(1.3372*$v0/$w,$w)>manycomponentsvariance(1.3372,$b,$r)){
    print "-";
  }
  if(abmperror(2**($r-1)*2.6744*$v0/$w,$r,$b,$w)>mrberror(2.6744)) {
    print "+";
  }
  if (abmpbberror(1.3372,$r,$b,$w)>mrberror(1.3372) or
      abmpbberror(2.6744,$r,$b,$w)>mrberror(2.6744)){
    print "<";
  }
  $rho=1.3372*$v0/$v;
  $lasterror=1000000000;
  while(abmperror($rho,$r,$b,$v)<$lasterror){
    $lasterror=abmperror($rho,$r,$b,$v);
    $rho=$rho*$increment;
  }
  $rhomin=$rhomax=$rho;
  while($rhomax/$rhomin<2){
    if(abmperror($rhomin,$r,$b,$v)>abmperror($rhomax,$r,$b,$v)){
      $rhomax*=$increment;
    } else {
      $rhomin/=$increment;
    }
  }
  $improvement=mrberror(2.6744)/abmperror($rhomin,$r,$b,$v);
  print "\t$improvement\n";
}
