#!/usr/bin/perl

# Computes rhominbig , rhobigbest and rhobigmax, the flow densities at
# which the big component takes over from the previous one, gives the
# best result and respectively hands it over to the next one.

$b=64;
$v=15063;
$r=9;

# The average (relative) error of the large component of the adaptive bitmap

sub abmperror($$$$){
  my ($rho,$r,$b,$v)=($_[0],$_[1],$_[2],$_[3]);
  my $v0=2*$b*(1-2**(-$r));
  my $rhomrbnext=$rho*$v/2**$r/$v0;
  return sqrt($v*(exp($rho)-1)+$b*(exp($rhomrbnext)-1)+2*$b*(exp($rhomrbnext/2)-1))/$v/$rho*$v0/$b/2;
}

# Computes the error of the multiresolution bitmap component just before the big virtual bitmap

sub abmpbberror($$$$){
  my ($rho,$r,$b,$v)=($_[0],$_[1],$_[2],$_[3]);
  my $v0=2*$b*(1-2**(-$r));
  return (sqrt($b*(exp($rho)-1)+$v*(exp($rho*$v0/2/$v)-1)+$b*2*(exp($rho/2**($r+1))-1))/$rho/$b/2);
}

sub mrberror($$){
  my ($rho,$b)=($_[0],$_[1]);
  my ($k)=2;
  return (sqrt((($k-1)/$k*(exp($rho)+exp($rho/$k)-2)+exp($rho/$k/$k)-1)*($k-1)/$k)/$rho/sqrt($b));
}


$increment=1.0001;
$v0=2*$b*(1-2**(-$r)); #equivalent to just extending the coarsest components

$rhob=1.3372*$v0/$v;

if(abmpbberror(1.3372,$r,$b,$v)>mrberror(1.3372,$b) or 
   abmpbberror(2.6744,$r,$b,$v)>mrberror(2.6744,$b) or 
   abmperror($rhob,$r,$b,$v)>mrberror(1.3372,$b)){
  die;
}

$rhoprev=2.6744;
while(abmperror($rhob,$r,$b,$v)<mrberror($rhoprev,$b)){
  $rhob=$rhob/$increment;
  $rhoprev=$rhoprev/$increment;
  if($rhoprev<1.3372){
    die;
  }
}

$rhob=$rhob*$increment;

print "Rhobigmin=$rhob\n";

$rhob=1.3372*$v0/$v;

$lasterror=1000000000;
while(abmperror($rhob,$r,$b,$v)<$lasterror){
  $lasterror=abmperror($rhob,$r,$b,$v);
  $rhob=$rhob*$increment;
}

print "Rhobigbest=$rhob\n";


$rhob=2**($r-1)*2.6744*$v0/$v;

if(abmperror($rhob,$r,$b,$v)>mrberror(1.3372,$b)){
  die;
}

$rhonext=1.3372;
while(abmperror($rhob,$r,$b,$v)<mrberror($rhonext,$b)){
  $rhob=$rhob*$increment;
  $rhonext=$rhonext*$increment;
  if($rhonext>2.6744){
    die;
  }
}

$rhob=$rhob/$increment;

print "Rhobigmax=$rhob\n";

